def is_single_band(img):
    first_pixel = img.getpixel((0, 0))
    return isinstance(first_pixel, int)


def compute_thredhold(img):
    thredhold = 350
    if is_single_band(img):
        thredhold = 120
        max_color = 0
        for w in range(0, img.size[0]):
            for h in range(0, img.size[1]):
                color = img.getpixel((w, h))
                if color > max_color:
                    max_color = color
        if max_color <= 127:
            thredhold = 50
    return thredhold


def make_imagestring(img, row_width=80):
    char_img = ""
    width = img.size[0]
    step = width / row_width
    if step <= 1:
        step = 1
    thredhold = compute_thredhold(img)

    for h in range(0, img.size[1], step):
        for w in range(0, img.size[0], step):
            color = img.getpixel((w, h))
            if isinstance(color, int):
                value = color
            else:
                value = sum(color)
            if value < thredhold:
                char_img += "*"
            else:
                char_img += " "
        char_img += "\n"
    return char_img



import string


def encode_charimage(charimg):
    charimg_lines = charimg.split("\n")
    encoded_lines = []
    for line in charimg_lines:
        if len(line) == 0:
            continue
        encoded_line = ""
        cursor = 0
        for index, char in enumerate(line):
            if index == 0:
                continue
            if index == len(line) - 1 and line[cursor] == char:
                encoded_line += "%s%s" % (line[cursor], index - cursor + 1)
                continue
            if char == line[cursor]:
                continue
            else:
                encoded_line += "%s%s" % (line[cursor], index - cursor)
                cursor = index
                if index == len(line) - 1:
                    encoded_line += "%s1" % line[cursor]
        encoded_lines.append(encoded_line)
    return "n".join(encoded_lines)


def decode_charimage(encoded_img, verbose=False):
    encoded_lines = encoded_img.split("n")
    lines = []
    for encoded_line in encoded_lines:
        parts = []
        is_digital = False
        cursor = 0
        for index, char in enumerate(encoded_line):
            char_is_digit = char in string.digits
            if index == 0:
                is_digital = char_is_digit
                continue
            if index == len(encoded_line) - 1 and char_is_digit == is_digital:
                parts.append(encoded_line[cursor:])
                continue
            if is_digital == char_is_digit:
                continue
            else:
                parts.append(encoded_line[cursor: index])
                cursor = index
                is_digital = not is_digital
                if index == len(encoded_line) - 1:
                    parts.append(char)
        if verbose:
            print parts
        line = ""
        step_cnt = len(parts) / 2
        for i in range(step_cnt):
            if verbose:
                print parts[i * 2: i * 2 + 2]
            line += parts[i * 2] * int(parts[i * 2 + 1])
        if verbose:
            print line
        lines.append(line)
    return "\n".join(lines)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
import argparse

from PIL import Image

from converter import make_imagestring
from encoder import encode_charimage, decode_charimage


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-w', dest='row_width', help="result row width", type=int, default=80)
    parser.add_argument('-e', dest='encode', help="encode the result", action='store_true')
    parser.add_argument('-d', dest='decode', help="decode the encoded_string", action='store_true')
    parser.add_argument('-v', dest='verbose', help="verbose output", action='store_true')
    parser.add_argument('image', help="<image_path/encoded_string>")
    args = parser.parse_args()

    if args.decode:
        encoded_image = raw_input("encodedm image:")
        print decode_charimage(encoded_image, args.verbose)
    else:
        i = Image.open(args.image)
        char_img = make_imagestring(i, args.row_width)
        if args.encode:
            print encode_charimage(char_img)
        else:
            print char_img


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages
setup(name='charimage',
      version="0.1.1-1",
      description="convert black-white image to string which can be displayed in terminal.",
      author="cong liu",
      author_email="shishusaiwai@vip.qq.com",
      license="GPL",
      packages=find_packages(),
      use_2to3=True,
      entry_points={"console_scripts": ["charimage = charimage.main:main"]})
